// Socket
// Base class for our SocketClient and SocketServer classes.
// Contains all the shared functionality of both clients and servers.
// Also this is the class instantiated when a SocketServer accepts a connection.

#ifndef __SOCKET_H__
#define __SOCKET_H__

#ifdef _WIN32
#include <winsock2.h>
#include <ws2tcpip.h>
#else
#include <netinet/in.h>
#include <netdb.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <arpa/inet.h>
#include <unistd.h>
// Declare a few things that Windows has but not others
#define INVALID_SOCKET -1
typedef int SOCKET;
#endif

#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

using namespace std;

// Structure for pass back info about our communication peer
struct PeerInfo 
{
	char name[INET6_ADDRSTRLEN];
	int  port;
};

class Socket
{
	public:
		virtual			~Socket();
		int				SendBytes(char* buffer, int len);
		int				RecvBytes(char* buffer, int maxlen);
		void			Close();
		PeerInfo		GetPeerInfo();

	protected:
		friend class SocketServer;
		friend class SocketSelect;

		Socket(int flags);
		Socket(SOCKET sock, int flags);

};

#endif
